import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class ldesign2 extends PApplet {

int numtypes = 3;
PFont font;

class Edge {
  PVector a, b;
  int type;
  
  Edge(PVector aa, PVector bb, int ttype) {
    a = aa; b = bb; type = ttype;
  }
  
  public PVector renderGetClosest(ArrayList rules, int levels, Frame f) {
    levels--;
    PVector xa = f.xform(a);
    PVector xb = f.xform(b);

    float dxa = xa.x - lastmx;
    float dya = xa.y - lastmy;
    float dxb = xb.x - lastmx;
    float dyb = xb.y - lastmy;
    a.z = dxa*dxa+dya*dya; b.z = dxb*dxb+dyb*dyb;
    if (dxa*dxa+dya*dya < dxb*dxb+dyb*dyb) {
      return a;
    } else {
      return b;
    }
  }
  
  public void render(ArrayList rules, int levels, Frame f) {
    levels--;
    PVector xa = f.xform(a);
    PVector xb = f.xform(b);
    Frame nexf = new Frame(xa,xb);
    if (levels <= 0) {
      line(xa.x,xa.y,xb.x,xb.y);
      float dgx = xa.x-gx, dgy = xa.y-gy;
      if (dgx*dgx+dgy*dgy < 25) {
        score();
      }
      dgx = xb.x-gx; dgy = xb.y-gy;
      if (dgx*dgx+dgy*dgy < 25) {
        score();
      }
      //println(xa.x + " " + xa.y + "  --  " + xb.x + " " + xb.y);
    } else {
      Rule r = (Rule)rules.get(type);
      for (int i = 0; i < r.edges.size(); i++) {
        Edge e = (Edge)r.edges.get(i);
//        Frame subf = f.getSubFrame(e.a, e.b);
        e.render(rules, levels, nexf);
      }
    }
  }
}

class Rule {
  ArrayList edges;
}

class Frame {
  PVector right, up, start;
  
  Frame(PVector a, PVector b) {
    start = a;
    up = PVector.sub(b, a);
    right = new PVector(up.y, -up.x);
  }
  
  public PVector xform(PVector p) {
    return PVector.add(start, 
            PVector.add(PVector.mult(right, p.x),
              PVector.mult(up, p.y)));
  }
  
  public Frame getSubFrame(PVector a, PVector b) {
    PVector ax = xform(a);
    PVector bx = xform(b);
    return new Frame(ax, bx);
  }
}

class System {
  ArrayList rules;
  PVector start, end;
  
  public void render(int firsttype, int levels) {
    Edge e = new Edge(new PVector(0,0), new PVector(0,1), firsttype);
    e.render(rules, levels, new Frame(start, end));
  }
  
  public PVector renderGetClosest(PVector start, PVector end, double space) {
    PVector closest = null;
    for (int i = 0; i < rules.size(); i++) {
      Rule r = (Rule)rules.get(i);
      for (int j = 0; j < r.edges.size(); j++) {
        Edge e = (Edge)r.edges.get(j);
        Frame f = new Frame(start, end);
        PVector c = e.renderGetClosest(null, 0, f);
        if (closest == null || c.z < closest.z) {
          closest = c;
        }
      }
      start.y += space;
    }
    return closest;
  }
  
  public void renderRules(PVector start, PVector end, double space) {
    for (int i = 0; i < rules.size(); i++) {
      Rule r = (Rule)rules.get(i);
      for (int j = 0; j < r.edges.size(); j++) {
        Edge e = (Edge)r.edges.get(j);
        Frame f = new Frame(start, end);
        e.render(null, 0, f);
      }
      start.y += space;
    }
  }
}

System s;

public void setup() {
  size(500,500);
  frameRate(30);

  score();
  font = loadFont("AgencyFB-Reg-48.vlw"); 
  textFont(font); 
    setupsystem();
}

public void setupsystem() {
    s = new System();
  //PVector aa = new PVector(0,0);
  //PVector bb = new PVector(.5,.5);
  //PVector cc = new PVector(0,1);
  PVector aa = new PVector(0,0);
  PVector bb = new PVector(0,.25f);
  PVector cc = new PVector(.5f,.5f);
  PVector dd = new PVector(0,.75f);
  PVector ee = new PVector(0,1);
  Edge ea = new Edge(aa,bb,0);
  Edge eb = new Edge(bb,cc,0);
  Edge ec = new Edge(cc,dd,0);
  Edge ed = new Edge(dd,ee,0);
  ArrayList edges = new ArrayList();
  edges.add(ea); edges.add(eb);
  edges.add(ec); edges.add(ed);
  Rule ra = new Rule();
  ra.edges = edges;
  s.rules = new ArrayList();
  s.rules.add(ra);
  s.start = new PVector(200, 250);
  s.end = new PVector(300, 250);
}

float lastmx, lastmy;
float gx, gy;
int scorec = -1;
int timeLeft = 100;
int levels = 7;
int stopscoring = 1;

public void score() {
  if (stopscoring == 1) {
    return;
  }
  scorec++;
  timeLeft = 100;
  gx = 100+random(300); gy = 100+random(300);
  levels++;
  if (levels > 15) {
    levels = 15;
  }
  setupsystem();
}

public void draw() {
  background(0);
  stroke(255);
  s.render(0, levels);
  s.renderRules(new PVector(200,100), new PVector(300,100), 50);
  
  float mx = mouseX, my = mouseY;
  
  if (mousePressed) {
    Frame f = new Frame(new PVector(0,0), new PVector(-1/100.0f,0));
    PVector v = s.renderGetClosest(
      new PVector(200,100), new PVector(300,100), 50);
    if (v.z < 100) { 
      PVector tr = f.xform(new PVector(mouseX-lastmx,mouseY-lastmy));
      v.x += tr.x;
      v.y += tr.y;
      v.z = 0;
    }
  }
  
  if (keyPressed && key == 'b') {
    stopscoring = 1;
  }
  if (keyPressed && key == 'v') {
    stopscoring = 0;
  }
  if (keyPressed && key == 'r') {
    setupsystem();
  }
  if (keyPressed && key == '=') {
    levels++;
    if (levels > 9) {
      levels = 9;
    }
  }
  if (keyPressed && key == '-') {
    levels--;
    if (levels < 2)
      levels = 2;
  }
  
  //ellipse(gx, gy, 5, 5);
  
  //text("score: " + scorec, 10, 50); 
  
  lastmx = mouseX; lastmy = mouseY;
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#ECE9D8", "ldesign2" });
  }
}
